/*
 * (C) 2008  Torsten Neuer <tneuer@inwise.de>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  $Id: guf.h,v 1.10 2008/06/16 16:34:37 tneuer Exp $
 *
 *  $Log: guf.h,v $
 *  Revision 1.10  2008/06/16 16:34:37  tneuer
 *  Added guf_filter_(get|set)_edge_mode () access functions
 *
 *  Revision 1.9  2008/06/11 17:52:06  tneuer
 *  Removed assertions not required anymore.
 *  Pointers to const data are const now, too.
 *
 *  Revision 1.8  2008/06/05 14:39:18  tneuer
 *  Added query functions for "Gimp::Date" and "Control %u::Step"
 *
 *  Revision 1.7  2008/06/05 13:46:32  tneuer
 *  Added set-value functions for "Gimp::Date" and "Control %u::Step"
 *
 *  Revision 1.6  2008/05/26 15:10:55  tneuer
 *  Minor code cleanup.
 *
 *  Revision 1.5  2008/05/23 15:26:20  tneuer
 *  uf2c now producing working native plug-ins from filter files
 *
 *  Revision 1.4  2008/05/16 14:02:52  tneuer
 *  Loading stand-alone (registered) filters.
 *
 *  Revision 1.3  2008/05/13 11:03:05  tneuer
 *  GUF compile fixes.
 *
 *  Revision 1.2  2008/05/05 13:32:40  tneuer
 *  New functions guf_filter_renew_from_pdb () and guf_filter_dump () for PDB operations.
 *
 *  Revision 1.1.1.1  2008/04/27 13:19:02  tneuer
 *  initial import
 *
 *  Revision 1.1  2008/04/22 15:25:38  tneuer
 *  Initial revision
 *
 */

#ifndef __GUF_H__
#define __GUF_H__

#include <glib.h>

#include <libgimp/gimp.h>

#define GUF_FLAGS (G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS)

/*
 *  GUF protocol levels.
 *
 *  Level 1 is the base level.
 *
 *  Greater levels are for future extensions to the .guf
 *  file format.  Applications need to specify the protocol
 *  level they need to understand.  Normally, this will be
 *  the current GUF_PROTOCOL.
 *
 *  The functions handling the .guf format will abort when
 *  an application attempts to operate on a .guf file with
 *  a higher protocol version than it could understand.
 *
 *  They will try to be smart with .guf files that have a
 *  lower protocol version.
 */

typedef enum {
  GUF_LEVEL_1 = 1
} GufProtocol;

#define GUF_PROTOCOL (GUF_LEVEL_1)

extern const gchar * const EXT_GUF;


/*
 *  These values are used to limit the lengths of
 *  certain string values for backward compatibility
 *  with old file formats.
 */

enum {
  F_LBL_LEN = 255,
  F_LBL_SIZE,
  F_SRC_LEN = 1023,
  F_SRC_SIZE,
};


/*
 *  We're hiding the GKeyFile behind a new class for
 *  type safety...
 */

typedef struct GKeyFile UserFilter;

#define guf_filter_new() ((UserFilter *)g_key_file_new ())

#define guf_filter_free(filter) g_key_file_free ((GKeyFile *)(filter))

gchar *
guf_filter_dump (const UserFilter * filter);

UserFilter *
guf_filter_new_from_data (const gchar * const data);


/*
 *  Write the internal filter representation back to disk.
 *
 *  This is a primitive function that does not check the
 *  integrity of the internal filter representation.
 *
 *  Returns TRUE if successful.
 */

gboolean
guf_write (const gchar * const filename,
	   const UserFilter *  filter);


/*
 *  Save a .guf file with leading comment using
 *  the specified protocol version.
 *
 *  Returns TRUE if successful.
 */

gboolean
guf_save (const gchar * const filename,
	  const gchar * const comment,
	  const GufProtocol   protocol,
	  const UserFilter *  filter);


/*
 *  Read a .guf file using the specified protocol level.
 *
 *  This is a primitive function that does not check the
 *  integrity of the internal filter representation.
 *
 *  Returns TRUE if successful.
 */

gboolean
guf_read (const gchar * const filename,
	  const GufProtocol   protocol,
	  UserFilter *        filter);


/*
 *  Load a .guf file using the specified protocol level.
 *
 *  Returns TRUE if successful.
 */

gboolean
guf_load (const gchar * const filename,
	  const GufProtocol   protocol,
	  UserFilter *        filter);


/*
 *  Change the registered flag in an existing .guf file
 *  with the specified protocol version to the value
 *  given.
 *
 *  Returns the protocol level of the filter that has
 *  been altered or GUF_ERROR.
 */

gboolean
guf_set_registered (const gchar * const filename,
		    const GufProtocol   protocol,
		    const gboolean      value);


/*
 *  Reset the filter to a default state.
 *
 *  If filter is not NULL, the old filter is destroyed.
 *  In any case, a new filter is created and filled with
 *  default values.
 */

UserFilter *
guf_filter_renew (UserFilter * filter);


/*
 *  Reload a filter from a Gimp PDB entry.
 *
 *  If filter is not NULL, the old filter is destroyed.
 *  In any case, a new filter is created and filled with
 *  PDB values.
 */

UserFilter *
guf_filter_renew_from_pdb (UserFilter *        filter,
			   const gchar * const data_key);


/*
 *  Reload a filter from the filter that a pdb-name points
 *  to.
 *
 *  If the filter is not NULL, the previous filter will be destroyed.
 *  The filter will be loaded from the file that is registered with
 *  the pdb-name.  If the load fails, it will be initialized to the
 *  default filter values.
 */

UserFilter *
guf_filter_renew_from_registry (UserFilter *        filter,
				const gchar * const rcfile,
				const gchar * const pdbname);


/*
 *  Primitive functions to get filter data from the internal
 *  representation.
 *
 *  Function names are self-explanatory.
 *  Functions returning (gchar *) will allocate a new memory
 *  region which must be g_free ()'d again.
 *
 *  The 'locale' functions will query the preferred locale
 *  and try to return the value in that locale if possible.
 */

GufProtocol
guf_filter_get_protocol (const UserFilter * filter);

gboolean
guf_filter_get_registered (const UserFilter * filter);

gchar *
guf_filter_get_title (const UserFilter * filter);

gchar *
guf_filter_get_locale_title (const UserFilter * filter);

gchar *
guf_filter_get_author (const UserFilter * filter);

gchar *
guf_filter_get_copyright (const UserFilter * filter);

gchar *
guf_filter_get_category (const UserFilter * filter);

gchar *
guf_filter_get_8bf_name (const UserFilter * filter);

gchar *
guf_filter_get_description (const UserFilter * filter);

gchar *
guf_filter_get_locale_description (const UserFilter * filter);

void
guf_filter_get_date (const UserFilter * filter,
		     guint *            year,
		     guint *            month,
		     guint *            day);

gchar *
guf_filter_get_date_string (const UserFilter * filter);

guint
guf_filter_get_date_year (const UserFilter * filter);

guint
guf_filter_get_date_month (const UserFilter * filter);

guint
guf_filter_get_date_day (const UserFilter * filter);

GimpPixelFetcherEdgeMode
guf_filter_get_edge_mode (const UserFilter * filter);

gint
guf_filter_get_version_major (const UserFilter * filter);

gint
guf_filter_get_version_minor (const UserFilter * filter);

gint
guf_filter_get_version_micro (const UserFilter * filter);

gchar *
guf_filter_get_version_string (const UserFilter * filter);

gboolean
guf_filter_get_map_enabled (const UserFilter * filter,
			    const guint        map);

gchar *
guf_filter_get_map_label (const UserFilter * filter,
			  const guint        map);

gchar *
guf_filter_get_locale_map_label (const UserFilter * filter,
				 const guint        map);

gchar *
guf_filter_get_map_tooltip (const UserFilter * filter,
			    const guint        map);

gchar *
guf_filter_get_locale_map_tooltip (const UserFilter * filter,
				   const guint        map);

gboolean
guf_filter_get_ctl_enabled (const UserFilter * filter,
			    const guint        ctl);

gchar *
guf_filter_get_ctl_label (const UserFilter * filter,
			  const guint        ctl);

gchar *
guf_filter_get_locale_ctl_label (const UserFilter * filter,
				 const guint        ctl);

gchar *
guf_filter_get_ctl_tooltip (const UserFilter * filter,
			    const guint        ctl);

gchar *
guf_filter_get_locale_ctl_tooltip (const UserFilter * filter,
				   const guint        ctl);

gint
guf_filter_get_ctl_preset (const UserFilter * filter,
			   const guint        ctl);

gint
guf_filter_get_ctl_step (const UserFilter * filter,
			 const guint        ctl);

gchar *
guf_filter_get_src (const UserFilter * filter,
		    const guint        channel);


/*
 *  Primitive functions to set filter data in the internal
 *  representation.
 *
 *  Function names are self-explanatory.
 *
 *  The "imported" functions will attempt conversion to utf-8
 *  characterset (used by non-native file readers).
 *
 *  The "locale" functions will set an entry for the specified
 *  locale value.
 */

void
guf_filter_set_protocol (UserFilter *      filter,
			 const GufProtocol value);

void
guf_filter_set_registered (UserFilter *   filter,
			   const gboolean value);

gchar *
guf_filter_set_title (UserFilter *        filter,
		      const gchar * const value);

gchar *
guf_filter_set_locale_title (UserFilter *        filter,
			     const gchar * const locale,
			     const gchar * const value);

gchar *
guf_filter_set_imported_title (UserFilter *        filter,
			       const gchar * const value);

gchar *
guf_filter_set_author (UserFilter *        filter,
		       const gchar * const value);

gchar *
guf_filter_set_imported_author (UserFilter *        filter,
				const gchar * const value);

gchar *
guf_filter_set_copyright (UserFilter *        filter,
			  const gchar * const value);

gchar *
guf_filter_set_imported_copyright (UserFilter *        filter,
				   const gchar * const value);

gchar *
guf_filter_set_category (UserFilter *        filter,
			 const gchar * const value);

gchar *
guf_filter_set_imported_category (UserFilter *        filter,
				  const gchar * const value);

gchar *
guf_filter_set_8bf_name (UserFilter *        filter,
			 const gchar * const value);

gchar *
guf_filter_set_imported_8bf_name (UserFilter *        filter,
				  const gchar * const value);

gchar *
guf_filter_set_description (UserFilter *        filter,
			    const gchar * const value);

gchar *
guf_filter_set_locale_description (UserFilter *        filter,
				   const gchar * const locale,
				   const gchar * const value);

void
guf_filter_set_date (UserFilter * filter,
		     const guint  year,
		     const guint  month,
		     const guint  day);

gchar *
guf_filter_set_date_string (UserFilter *        filter,
			    const gchar * const date);

GimpPixelFetcherEdgeMode
guf_filter_set_edge_mode (UserFilter *             filter,
			  GimpPixelFetcherEdgeMode value);

void
guf_filter_set_version_major (UserFilter * filter,
			      const gint   value);

void
guf_filter_set_version_minor (UserFilter * filter,
			      const gint   value);

void
guf_filter_set_version_micro (UserFilter * filter,
			      const gint   value);

void
guf_filter_set_version_all (UserFilter * filter,
			    const gint   major,
			    const gint   minor,
			    const gint   micro);

gboolean
guf_filter_set_map_enabled (UserFilter *   filter,
			    const guint    map,
			    const gboolean value);

gchar *
guf_filter_set_map_label (UserFilter *        filter,
			  const guint         map,
			  const gchar * const value);

gchar *
guf_filter_set_locale_map_label (UserFilter *        filter,
				 const guint         map,
				 const gchar * const locale,
				 const gchar * const value);

gchar *
guf_filter_set_imported_map_label (UserFilter *        filter,
				   const guint         map,
				   const gchar * const value);

gchar *
guf_filter_set_map_tooltip (UserFilter *        filter,
			    const guint         map,
			    const gchar * const value);

gchar *
guf_filter_set_locale_map_tooltip (UserFilter *        filter,
				   const guint         map,
				   const gchar * const locale,
				   const gchar * const value);

gboolean
guf_filter_set_ctl_enabled (UserFilter *   filter,
			    const guint    ctl,
			    const gboolean value);

gchar *
guf_filter_set_ctl_label (UserFilter *        filter,
			  const guint         ctl,
			  const gchar * const value);

gchar *
guf_filter_set_locale_ctl_label (UserFilter *        filter,
				 const guint         ctl,
				 const gchar * const locale,
				 const gchar * const value);

gchar *
guf_filter_set_imported_ctl_label (UserFilter *        filter,
				   const guint         ctl,
				   const gchar * const value);

gchar *
guf_filter_set_ctl_tooltip (UserFilter *  filter,
			    const guint   ctl,
			    const gchar * value);

gchar *
guf_filter_set_locale_ctl_tooltip (UserFilter *        filter,
				   const guint         ctl,
				   const gchar * const locale,
				   const gchar * const value);

void
guf_filter_set_ctl_preset (UserFilter * filter,
			   const guint  ctl,
			   const gint   value);

void
guf_filter_set_ctl_step (UserFilter * filter,
			 const guint  ctl,
			 const gint   value);

gchar *
guf_filter_set_src (UserFilter *  filter,
		    const guint   channel,
		    const gchar * value);

gchar *
guf_filter_set_imported_src (UserFilter *        filter,
			     const guint         channel,
			     const gchar * const value);

#endif /* __GUF_H__ */
